@@ -29,28 +29,42 @@
 // TODO: fill out all fields of https://doc.rust-lang.org/cargo/reference/manifest.html
 // TODO: implement change_winsize
 
-#![deny(warnings)]
+#![warn(bad_style)]
+#![warn(future_incompatible)]
+#![warn(rust_2018_compatibility)]
+#![warn(rust_2018_idioms)]
+#![warn(unused)]
 
-#![warn(anonymous_parameters)]
+#![warn(bare_trait_objects)]
 #![warn(missing_copy_implementations)]
 #![warn(missing_debug_implementations)]
 #![warn(missing_docs)]
+#![warn(single_use_lifetimes)]
 #![warn(trivial_casts)]
 #![warn(trivial_numeric_casts)]
 #![warn(unreachable_pub)]
 #![warn(unstable_features)]
-#![warn(unused_extern_crates)]
 #![warn(unused_import_braces)]
+#![warn(unused_lifetimes)]
 #![warn(unused_qualifications)]
 #![warn(unused_results)]
 #![warn(variant_size_differences)]
 
-// this library is fundamentally built upon unsafe code
+// this entire crate is unsafe code
 #![allow(unsafe_code)]
 
 #![cfg_attr(feature="cargo-clippy", warn(clippy))]
+#![cfg_attr(feature="cargo-clippy", warn(clippy_complexity))]
+#![cfg_attr(feature="cargo-clippy", warn(clippy_correctness))]
 #![cfg_attr(feature="cargo-clippy", warn(clippy_pedantic))]
-#![cfg_attr(feature="cargo-clippy", allow(similar_names))]
+#![cfg_attr(feature="cargo-clippy", warn(clippy_perf))]
+#![cfg_attr(feature="cargo-clippy", warn(clippy_style))]
+
+// TODO: we can remove `bindgen` as a direct dependency and just bundle
+// its output since it's static; these should pass much more reliably
+// then
+//
+// #![cfg_attr(feature="cargo-clippy", warn(clippy_cargo))]
 
 extern crate libc;
 
@@ -91,8 +105,8 @@ sudo_io_plugin! {
         close:      close,
         log_ttyout: log_ttyout,
         log_stdin:  log_disabled,
-        log_stdout: log_ttyout,
-        log_stderr: log_ttyout,
+        log_stdout: log_stdout,
+        log_stderr: log_stderr,
      }
 }
 
@@ -163,6 +177,30 @@ impl SudoPair {
     }
 
     fn log_ttyout(&mut self, log: &[u8]) -> Result<()> {
+        if !self.plugin.command_info.iolog_ttyout {
+            return Ok(())
+        }
+
+        self.log_output(log)
+    }
+
+    fn log_stdout(&mut self, log: &[u8]) -> Result<()> {
+        if !self.plugin.command_info.iolog_stdout {
+            return Ok(())
+        }
+
+        self.log_output(log)
+    }
+
+    fn log_stderr(&mut self, log: &[u8]) -> Result<()> {
+        if !self.plugin.command_info.iolog_stderr {
+            return Ok(())
+        }
+
+        self.log_output(log)
+    }
+
+    fn log_output(&mut self, log: &[u8]) -> Result<()> {
         // if we have a socket, write to it
         self.socket.as_mut().map_or(Ok(()), |socket| {
             socket
@@ -302,6 +340,11 @@ impl SudoPair {
             return true;
         }
 
+        // policy plugins can inform us that logging is unnecessary
+        if self.is_exempted_from_logging() {
+            return true;
+        }
+
         // exempt if the user who's sudoing is in a group that's exempt
         // from having to pair
         if self.is_sudoing_from_exempted_gid() {
@@ -314,11 +357,6 @@ impl SudoPair {
             return true;
         }
 
-        // exempt if the approval command is the command being invoked
-        if self.is_sudoing_approval_command() {
-            return true;
-        }
-
         false
     }
 
@@ -349,6 +387,22 @@ impl SudoPair {
         false
     }
 
+    ///
+    /// Returns true if the policy plugin has not given us any
+    /// facilities to log output for.
+    ///
+    fn is_exempted_from_logging(&self) -> bool {
+        if
+            !self.plugin.command_info.iolog_ttyout &&
+            !self.plugin.command_info.iolog_stdout &&
+            !self.plugin.command_info.iolog_stderr
+        {
+            return true;
+        }
+
+        false
+    }
+
     fn is_sudoing_to_user_and_group(&self) -> bool {
         // if a user is doing `sudo -u ${u} -g ${g}`, we don't have a
         // way to ensure that the pair can act with permissions of both
@@ -379,10 +433,6 @@ impl SudoPair {
         )
     }
 
-    fn is_sudoing_approval_command(&self) -> bool {
-        self.plugin.command_info.command == self.options.binary_path
-    }
-
     fn is_sudoing_to_user(&self) -> bool {
         self.plugin.user_info.uid != self.plugin.command_info.runas_euid
     }
@@ -569,6 +619,9 @@ impl PluginOptions {
     }
 }
 
+// TODO: single_use_lifetimes was committed, but I'm not sure there's
+// actually a way to satisfy the linter for the time being
+#[allow(single_use_lifetimes)]
 impl<'a> From<&'a OptionMap> for PluginOptions {
     fn from(map: &'a OptionMap) -> Self {
         Self {
